package com.iRecruit.modules.auth.controller;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.iRecruit.annotation.AuditLogs;
import com.iRecruit.constant.LeaderCalcConfig;
import com.iRecruit.constant.StatusCode;
import com.iRecruit.exception.ResourceNotFoundException;
import com.iRecruit.modules.auth.model.L1slabsMapping;
import com.iRecruit.modules.auth.model.L2slabsMapping;
import com.iRecruit.modules.auth.payload.L1slabsDto;
import com.iRecruit.modules.auth.payload.L2slabsDto;
import com.iRecruit.modules.auth.payload.request.ApiResponse;
import com.iRecruit.modules.auth.service.L2slabsMappingService;


import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController 
@RequestMapping("/api/v1")
@Api(description = "L2 Slabs maping management API")

public class L2slabsMappingController {

	@Autowired
	  L2slabsMappingService l2slabsMappingService;
	
	LeaderCalcConfig LeaderCalcConfig;
  @PostMapping("/l2salbsmapping/get")
  @ApiOperation("Get all l2slabs mapping list")
  @AuditLogs(isRequest = true, isResponse = true,service="Get l2slabs")
  
//Neel – 20-Apr-2020 – Tata - l2 slabs mapping – Added function for fetch all data of l2 slabs from table
  public ResponseEntity<?> getAllL1slabsMappings( ){
  	return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "l2 slabs List", l2slabsMappingService.findAll().stream().map(L2slabsMapping -> new L2slabsDto(L2slabsMapping))), HttpStatus.OK);
  }
  
  @PostMapping("/l2salbsmapping/create")
  @ApiOperation("Create l2salbsmapping")
  @AuditLogs(isRequest = true, isResponse = true,service="Create l2salbsmapping")
  
  //Neel – 20-Apr-2020 – Tata - l2 slabs mapping – Added function for store all data of l1 slabs  table
  public ResponseEntity<?> createL1slabsmapping(@RequestBody L2slabsDto l2slabsmapping) {
  	System.out.println("in l2salbsmapping contrroler");
  	System.out.println("in l2salbsmapping");
  	L2slabsMapping l2 = null;
  	try {
  		l2 =  l2slabsMappingService.save(l2slabsmapping,true);

  	}catch(ResourceNotFoundException e) {
  		e.printStackTrace();
  		return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_NOT_FOUND, true, "l2mapping Not found", l2.getId()), HttpStatus.NOT_FOUND);
  	}catch(Exception e) {
  		e.printStackTrace();
  		return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true, "l2mapping creation failed", l2.getId()), HttpStatus.BAD_REQUEST);
  	}
  	return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "l2mapping added successfully", new L2slabsDto(l2)), HttpStatus.OK);
  	//return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_EXIST, true, "USER_Already_Exist", null), HttpStatus.NOT_FOUND);
  }
  
  @PostMapping("/l2salbsmapping/update")
  @ApiOperation("update l2salbsmapping")
  @AuditLogs(isRequest = true, isResponse = true,service="Create l2salbsmapping")
  public ResponseEntity<?> updateL1slabsmapping(@RequestBody L2slabsDto l2slabsmapping) {
  	System.out.println("in l2salbsmapping contrroler");
  	System.out.println("in l2salbsmapping");
  	L2slabsMapping l2 = null;
  	try {
  		l2 =  l2slabsMappingService.save(l2slabsmapping,false);

  	}catch(ResourceNotFoundException e) {
  		e.printStackTrace();
  		return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_NOT_FOUND, true, "l2mapping Not found", l2.getId()), HttpStatus.NOT_FOUND);
  	}catch(Exception e) {
  		e.printStackTrace();
  		return new ResponseEntity(new ApiResponse(StatusCode.UNEXPECTED_ERROR, true, "l2mapping updation failed", l2.getId()), HttpStatus.BAD_REQUEST);
  	}
  	return new ResponseEntity(new ApiResponse(HttpStatus.OK.value(), true, "l2mapping added successfully", new L2slabsDto(l2)), HttpStatus.OK);
  	//return new ResponseEntity(new ApiResponse(StatusCode.RESOURCE_EXIST, true, "USER_Already_Exist", null), HttpStatus.NOT_FOUND);
  }
}
