package com.iRecruit.security;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import javax.transaction.Transactional;

import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import com.iRecruit.modules.auth.model.User;
import com.iRecruit.modules.auth.payload.request.LoginRequest;

import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

@Getter
@Setter
@ToString
public class UserPrincipal implements UserDetails {

    private String name;

    private String username;
    
    private String role;

    private String email;

    private String deviceInfo;
    
    private String browserInfo;
    
    private String OS;
    
    private String mobileno;

    private Collection<? extends GrantedAuthority> authorities;

    
    public UserPrincipal(String name, String username, String email,String deviceInfo,String OS,String browserInfo,String mobileno, Collection<? extends GrantedAuthority> authorities) {
        this.name = name;
        this.username = username;
        this.email = email;
        this.mobileno = mobileno;
        this.OS = OS;
        this.browserInfo = browserInfo;
        this.deviceInfo = deviceInfo;
        this.authorities = authorities;
    }
    
    public UserPrincipal(String name,String username, String email,String deviceInfo,String OS,String BrowserInfo,String mobileno, Collection<? extends GrantedAuthority> authorities, Set<String> roleType) {
    	this.name = name;
        this.username = username;
        this.email = email;
        this.deviceInfo = deviceInfo;
        this.mobileno = mobileno;
        this.authorities = authorities;
        this.OS = OS;
        this.browserInfo = browserInfo;
    }
    public UserPrincipal() {}

    @Transactional
    public static UserPrincipal create(User user,LoginRequest loginRequest) {
    	
    	List<String> roles = new ArrayList<String>();
    	roles.add(user.getRole().name());
        List<GrantedAuthority> authorities = roles.stream().map(role ->
                new SimpleGrantedAuthority("ROLE_"+role)
        ).collect(Collectors.toList());

        
        return new UserPrincipal(user.getFirstName(), user.getId(), user.getEmailId(),loginRequest.getDeviceInfo(),loginRequest.getOS(),loginRequest.getBrowserInfo(), user.getMobileNo(), authorities);
    }
    
    
    /*public static UserPrincipal create(User user, Set<String> roleType) {
        List<GrantedAuthority> authorities = user.getRoles().stream().map(role ->
                new SimpleGrantedAuthority("ROLE_"+role.getName())
        ).collect(Collectors.toList());

        return new UserPrincipal(
                user.getId(),
                user.getFName(),
                user.getUserId(),
                user.getEmail(),
                user.getPassword(),
                user.getMobileNumber(),
                authorities,
                roleType
        );
    }*/

    public String getDeviceInfo() {
    	return this.deviceInfo;
    }
    public String getOS() {
    	return this.OS;
    }
    public String getBrowserInfo() {
    	return this.browserInfo;
    }
    @Override
    public String getPassword() {
       return null;
    }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return authorities;
    }

    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    @Override
    public boolean isAccountNonLocked() {
        return true;
    }

    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @Override
    public boolean isEnabled() {
        return true;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        UserPrincipal that = (UserPrincipal) o;
        return Objects.equals(username, that.username);
    }

    @Override
    public int hashCode() {

        return Objects.hash(username);
    }

	@Override
	public String getUsername() {
		// TODO Auto-generated method stub
		return username;
	}
}